
# q-expansion of newform 2016.4.a.bb, downloaded from the LMFDB on 08 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2016
weight = 4
poly_data = [244, 58, -41, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[78, 39, -5, -1], 7], [[-248, 128, 8, -4], 7], [[-173, -73, 11, 3], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [-2, 1, 0, 0], [7, 0, 0, 0], [-6, -1, -1, 0], [5, -1, 0, 1], [3, 2, -1, -1], [5, 1, 2, -1], [-39, 0, 1, 1], [-34, -4, 3, -2], [17, 9, 2, 3], [59, -15, -2, -1], [-81, -12, -3, -1], [-30, -6, 2, -2], [-90, 4, 2, -2], [-30, 22, 5, 6], [-100, -42, 6, -8], [123, -31, -4, -1], [140, 28, -14, -4], [-41, 44, -5, 7], [-100, -42, -12, 2], [-50, -10, -16, 2], [176, -20, -12, 8], [-423, -2, -9, 13], [-320, -38, 24, 6], [-636, -39, 20, -10], [-177, -41, 2, -19], [160, -71, 11, 26], [88, 54, 16, 18], [-560, 28, 24, -24], [-412, 108, 16, 20], [508, -28, 0, -28], [-1274, -28, 8, -18], [518, -42, 32, 18], [-1390, 30, -9, -10], [-486, 82, -8, -26], [-567, 23, -68, -7], [956, 12, 46, -20], [-18, 128, 46, 30], [-1692, -15, -34, -10], [-774, -47, -3, -12], [223, 265, 4, -9], [365, -42, -49, 49], [-121, 45, 58, -57], [-2322, 6, 33, 50], [-1320, 128, 52, 0], [780, 268, -18, -20], [-942, -222, -56, 14], [-488, 222, -54, 4], [-475, -89, -8, 25], [-2974, -228, 2, 10], [-619, -288, -11, -3], [-34, -40, -92, 32], [-864, -234, -70, -4], [-3575, 6, 59, -19], [-521, -102, 77, -45], [-3538, -135, 22, 48], [-859, -211, 14, 31], [1337, -37, -2, 77], [-3962, -56, -162, -26], [2187, 415, -54, 17], [-3466, 207, 36, -76], [3819, -289, -26, -95], [-110, 160, -30, 34], [978, -224, -12, -40], [-4314, 470, 41, -22], [2880, -432, 22, -24], [-2, -536, 0, 32], [-1480, -33, -99, 46], [291, -407, -24, -121], [-6029, -90, -71, -57], [-2127, -78, -105, -83], [-4376, 256, -36, -128], [1438, -416, -24, 56], [2116, 1028, 142, 36], [4376, -536, -48, 40], [-4258, -132, 69, 110], [-983, 295, 36, 105], [-1082, -320, 104, -26], [2498, -284, 148, 68], [1116, 582, -242, -40], [3073, 155, -162, 5], [-447, -126, -1, 125], [-2914, 436, -60, 148], [-6630, -710, 164, -42], [-112, 195, 97, -210], [-2144, -132, -6, 104], [-454, -44, 216, -132], [-3894, 267, 78, 48], [-6988, 764, 60, -236], [2604, 246, 134, 168], [-822, -876, 230, 98], [-9094, -318, -168, 6], [4286, -259, -27, -48], [10218, -62, -78, 102], [-1348, 976, 236, -68], [2944, -401, -230, 110], [-1221, 810, 283, -89], [5560, -680, -96, -104], [969, 1483, 22, -75], [7394, 650, -262, 94], [2998, -106, 39, -246], [-4920, -250, -102, 36], [450, -1464, 20, -14], [4614, -18, 6, -278], [2290, 1648, 232, -16], [-2508, -714, 6, 48], [-235, 296, -127, -275], [-9969, 1156, 155, 175], [-3370, 444, -320, 20], [-3610, -106, -328, 234], [-1100, 1102, -168, -38], [9626, 192, 402, -22], [3330, 274, 500, -106], [-12268, 380, 72, 340], [1786, -4, -316, -206], [11033, -571, 198, 107], [-9394, 556, -126, -10], [8098, 1376, 197, -38], [3716, 517, 163, 150], [-5135, -489, 480, -151], [-3915, -373, 46, -127], [3424, -1215, 198, 146], [-7484, -1537, -103, 90], [16156, 1036, 180, 20], [9262, 1648, 147, 22], [-4336, -202, 312, 178], [13936, 1060, 28, 88], [-13109, -477, 250, 113], [-1291, -481, -172, 113], [11076, 244, 86, -108], [7201, -250, 435, -155], [-6152, -480, 64, -224], [1176, 94, 32, 218], [15069, 386, -447, 105], [4098, -792, 372, -208], [13760, -671, 120, -286], [16514, -382, -336, -106], [18004, 77, 294, 94], [7424, 432, 718, 208], [804, 1236, -352, 348], [4318, 2230, 201, 162], [-14542, -86, -148, -130], [11066, -1165, -269, -56], [-801, 369, -520, 399], [3810, -2216, 90, 290], [10575, 93, -640, 275], [6445, 946, -563, -135], [-2947, -7, 18, 199], [-6637, 774, 373, 551], [-12392, -1494, -200, -386], [9897, -1452, 109, -263], [9186, -2598, -190, 190], [22834, 692, 174, -182], [-1890, -378, -338, 2], [14917, -290, -73, 441], [-8350, 1914, -180, -178], [7155, 3160, 213, 75], [-4756, -190, 124, -554], [20604, -1741, -40, -126], [-24422, 607, -377, 416], [10696, 504, -614, 296], [-10816, -1368, -468, -136], [39396, 420, -72, -212], [29982, 2348, -172, 22], [-18728, -1932, -460, -96], [12106, -1230, -80, 406], [-22923, -469, -488, 37], [-5811, 1671, 414, 101], [24074, -389, 166, 16], [-38852, 1797, 347, -498], [11953, 2543, 652, 209], [-8373, -3320, -29, -13], [-14861, 893, 410, 623], [15280, 200, -8, -120], [23385, -1868, -43, -471], [-5798, -2414, 506, -490], [6814, -1608, -47, 422], [-6227, 3285, 98, -473], [4697, 2455, 24, -23], [13426, -294, -164, 766], [10912, 1780, 900, 296], [-819, 1519, 122, 73], [8405, -384, 877, -67], [-28095, 532, -1011, 81], [-3026, -1133, -286, 388], [15250, 2760, -8, 32], [9556, 276, -172, 44], [-1793, 1633, -950, -525], [-14360, -768, 1000, -472], [4450, 1844, -592, 140], [12142, 2779, -977, 108], [-172, 1444, 542, -828], [-10458, -2205, -394, -324], [-24802, 603, 383, 732], [12025, 872, -623, -175], [-9947, 2739, 790, 193], [-2271, 2043, 146, -707], [-11493, 80, 483, 467], [56486, -2480, -394, 630], [13726, -374, -197, -358], [24994, 2210, -1028, 718], [-8617, 2753, -784, 47], [5924, 630, -628, -446], [15380, 1536, 300, -396], [3774, 1431, -36, 396], [38190, 1870, -412, 210], [-31634, -2001, -253, -268], [-28128, 2948, -80, -280], [-20843, -2287, -606, -561], [15510, -1916, 128, 252], [10516, 2799, 788, -462], [-2674, -4522, -1304, 18], [50116, 2828, -88, -228], [-8278, -4848, -1302, -38], [5912, -570, 120, 74], [28854, -1106, 376, -390], [-28103, 1112, 791, -287], [-39809, 2574, 321, -349], [566, 1908, 1345, 150], [-20044, 970, -528, 798], [-17517, 187, -266, 9], [-9512, -2272, 234, -920], [43128, 3776, -688, -48], [26196, -1732, 344, -724], [36323, 185, 174, 423], [-46993, -3912, -613, -137], [-38177, 446, 97, 995], [13, 4981, -510, 167], [-35900, 3519, 113, -998], [6320, -1010, 1368, 250], [-3301, -2945, 498, 913], [38590, -2362, -192, -478], [3268, 872, -1284, 532], [-36077, -137, 386, -391], [-28972, 3564, 712, 1516], [-1112, -238, 928, -1074], [-38302, 118, 559, 1110], [17900, 51, -627, -118], [31272, -1220, -180, -1088], [-43528, -1965, 809, -1034], [-57048, 1560, 16, 88], [-15162, -1610, 1076, -22], [-19701, -1183, -94, -409], [3125, 2130, -781, 1265], [75102, 1456, -530, -82], [51281, 1257, 1374, -141], [-13148, -1710, -1398, 760], [-37126, 34, -122, -410], [26469, 1940, -471, -1291], [12954, 1088, -1624, -808], [-31755, 2644, -499, 1477], [-59083, -5958, 731, 193], [-10540, 110, -1660, 106], [-63840, 665, 368, 562], [-45140, -2407, -857, -634], [31429, -3129, -590, 257], [-48279, 4165, 18, 91], [-25886, 707, -1832, 296], [18684, 1162, -184, 838], [40510, -1082, -372, -1390], [25506, -3147, 1441, 388], [-44027, 3943, 1060, -951], [12231, -3715, -556, -1181], [20477, -4712, -457, -539], [-22858, -394, -268, 146], [-66542, -1106, 167, -842], [-72441, -3606, 1145, -797], [-56022, -1998, -662, -714], [-1378, -1488, 801, 470], [-4867, 4683, -1332, 1797], [3104, -992, -134, -1544], [87660, 3910, 140, 66], [54828, 964, -656, -1124], [-238, 4852, 408, -20], [9019, -3869, -58, -639], [-50340, 2277, 1347, -146], [20199, 2221, -2110, 1227], [21759, 1701, 1482, -457], [89856, -370, -246, -436], [-26376, 412, 1036, -944], [-7868, 4060, -2272, 1844], [40620, 5168, 572, 556], [11309, -1341, -1656, -1267], [-32244, 204, 696, -1388], [-57311, -622, 383, -3], [-57218, 2360, -160, -784], [46130, -4270, -1705, 606], [-12475, 3133, -1714, 1007], [-62759, 3698, 331, -1323], [-78746, 1706, -1385, 1346], [-78864, -1872, -904, -208], [22398, 7092, -378, 918], [9896, -1178, -1118, -892], [16674, 6556, -696, 1260], [1812, 8677, 246, 462], [48976, -1528, 60, -952], [-58806, 2061, 878, 692], [107072, 1702, 1866, -4], [-5258, -6282, 1940, 482], [-9804, 290, -1192, -1538], [9642, -4828, 2213, 10], [89076, -2580, -948, 644], [69242, 3987, -557, -1016], [-63018, -4226, -834, -566], [-18102, 1740, -1232, -396], [-48214, 4177, -1931, -732], [39053, 10231, 748, 681], [-54891, 750, -1617, 1161], [-4933, 2697, -6, 447], [126392, 1588, 724, -192], [-32854, 3316, -1933, -934], [-65394, 5152, 340, 942], [26462, 118, 1182, 258], [72367, -3754, -1335, 739], [68012, 2342, -412, 1362], [34776, 3620, -1804, 1856], [-64923, -4784, 317, -51], [-64012, -3242, -500, 1202], [28326, -7408, -1006, 1046], [48715, 10751, 1122, 49], [42521, 5623, -2026, -187], [7346, -1853, 2316, 1048], [75610, 10082, -1080, 326], [-40224, -2568, -1862, -800], [41894, 192, -1516, -840], [-36360, 5048, -1946, 928], [-40299, 2737, 1278, 1983], [-41373, -8156, 959, 99], [1802, -6760, -1695, 18], [-60690, -5036, 2040, -1660], [89738, -2008, 1233, -1358], [68829, -4283, 322, -1737], [-27800, 154, -402, -2292], [-83948, -6284, 1486, -828], [51472, -142, -2058, -796], [88820, -2506, 1944, -478], [60165, 1638, -813, -215], [-49648, -6438, 2724, -1506], [110169, -5759, -1054, 1547], [80546, -1596, 1176, 420], [-14242, 7968, -40, 2270], [33670, -623, -98, 696], [-65404, 1468, 1424, -2284], [-23882, -6952, 403, 334], [40106, 13919, 1711, 1008], [-103113, 2209, -1244, 1615], [-11642, -10482, 2130, 2202], [50004, 1800, 396, 836], [57100, 11637, -1096, -682], [48221, -9127, -1314, -137], [-47099, -4569, 2126, 661], [73142, 8362, -315, -798], [1130, 5434, -1724, 1446], [-3006, -9388, -2136, -252], [-355, 832, 2605, -315], [123164, -2512, -1204, 412], [-89656, 1000, -592, 88], [110964, 10192, -622, -284], [7011, -128, 1307, -1701], [47334, -4584, 2680, 768], [-8353, -10380, -1841, -1857], [-57159, -10926, 855, -875], [-55980, -2255, 1487, 606], [-15164, 2164, -1868, -1860], [98993, 4823, -210, -35], [55182, -7067, -1746, -1176], [-36758, -8494, -3136, 790], [-63997, 3297, 290, -2349], [-117036, 3735, 1553, 1178], [-70989, -15497, -474, -407], [-24699, -1062, -4317, 945], [-40362, -13566, 1085, -910], [-112595, -1211, -1158, -553], [-89507, 3199, -770, 1617], [-68708, 5548, 816, 1876], [52431, -13842, -2143, 1395], [107734, 164, -1320, -868], [-19635, -728, -321, 2325], [-17120, -4606, -416, 1166], [-26424, -67, -1602, -2382], [77116, 5668, 1318, -140], [-28848, 9720, 648, -1880], [74544, -6948, 2524, 904], [61132, -340, -2576, -1100], [-49224, 1464, -1296, 40], [25310, 10326, 2536, 978], [-154703, -893, -676, -35], [-85462, 2866, 530, -2586], [-13038, -5612, 1918, 362], [-54497, 6001, 3314, -2981], [-28294, 11473, 2792, 1628], [-50448, -11723, -337, -766], [102595, 583, -1598, -983], [-74145, 8636, 571, -1009], [-32998, -1732, 2624, 604], [85944, -4640, 104, -560], [-27571, 2568, -4177, 2965], [25186, 8442, 690, 2558], [140874, 1516, -2179, 634], [1145, -6617, 3832, -1255], [30264, 13596, 1862, -160], [-22856, -864, 188, 3104], [205811, -800, 1181, -101], [5654, -727, -886, 3992], [-52592, -15368, 1244, 472], [20761, -493, 1310, -2747], [21436, -7176, 2230, -2180], [184962, 3202, 244, 358], [152164, 62, -2354, 1168], [43992, -2730, -2148, 2402], [61134, -292, 809, 1342], [40394, -6047, 1861, -2844], [73754, -10494, 570, 230], [-79117, 1437, 1882, 3791], [34662, -5095, -4094, 2504], [-13863, -8420, 2677, -2791], [-94533, 11955, 2618, -1311], [97529, 5244, -2969, -23], [117606, 6964, -30, 2078], [-48814, 10988, 1889, 354], [-98349, 65, -284, -1441], [86079, 11028, 771, 1055], [34500, -1956, 1112, -292], [-69616, 13390, -928, -1774], [166610, 8703, -528, -368], [-73764, -2225, 1745, -622], [-5574, 15968, -242, 906], [-45720, 264, -228, 3992], [-38686, 9956, 1262, 154]]
